﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
    
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using XNAGraphics = Microsoft.Xna.Framework.Graphics;
using System.IO;
    
namespace Application2
{
    public partial class MainForm : Form
    {
        // Вспомогательные поля
        const string effectFileName = "Data\\ColorFill.fx"; 
        GraphicsDevice device = null;
        PresentationParameters presentParams = new PresentationParameters();
        Effect effect = null;
        VertexDeclaration decl = null;
        VertexPositionColor[] vertices = new VertexPositionColor[3];
        bool closing = false;

        public MainForm()
        {
            InitializeComponent();
    
            // Создание устройства и эффекта
            CreateDeviceAndEffect();
        }
    
        void CreateDeviceAndEffect()
        {
            // Отключить у панели автоматическую очистку фона и задать
            // автоматический вызов события Paint при изменении размеров 
            xnaPanel1.SetStyle(ControlStyles.Opaque | ControlStyles.ResizeRedraw, true);

            // Настраиваем объект представления через его свойства
            presentParams.IsFullScreen = false; // Включаем оконный режим
            presentParams.BackBufferCount = 1;  // Включаем задний буфер 
            // для двойной буферизации 
            // Переключение переднего и заднего буферов 
            // должно осуществляться с максимальной эффективностью
            presentParams.SwapEffect = SwapEffect.Discard;
            // Устанавливаем размеры заднего буфера по клиентской области панели
            presentParams.BackBufferWidth = xnaPanel1.ClientSize.Width;
            presentParams.BackBufferHeight = xnaPanel1.ClientSize.Height;

            // Создадим графическое устройство с заданными настройками
            device = new GraphicsDevice(GraphicsAdapter.DefaultAdapter, DeviceType.Hardware,
                xnaPanel1.Handle, presentParams);

            // Создаем декларацию о формате применяемого хранилища вершин
            decl = new VertexDeclaration(device, VertexPositionColor.VertexElements);

            // Загружаем и компилируем эффект из файла 
            CompiledEffect compiledEffect;
            try
            {
                compiledEffect = Effect.CompileEffectFromFile(effectFileName,
                    null, null, CompilerOptions.None, TargetPlatform.Windows);
            }
            catch (IOException ex)
            {
                closing = true;
                MessageBox.Show(ex.Message, "Ошибка при загрузке эффекта",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Idle += new EventHandler(Application_Idle);
                return;
            }
            // Проверяем, нормально ли откомпилирован эффект 
            if (!compiledEffect.Success)
            {
                closing = true;
                MessageBox.Show(String.Format(
                    "Ошибка при компиляции эффекта:\n{0}", compiledEffect.ErrorsAndWarnings),
                    "Критическая ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Idle += new EventHandler(Application_Idle);
                return;
            }

            // Создаем объект эффекта
            effect = new Effect(device, compiledEffect.GetEffectCode(),
                CompilerOptions.NotCloneable, null);
            // Проверяем, поддерживается ли видеокартой указанная в эффекте техника
            if (!effect.CurrentTechnique.Validate())
            {
                closing = true;
                MessageBox.Show(String.Format("Ошибка при проверке техники \"{0}\" эффекта \"{1}\"\n" +
                    "Скорее всего, функциональность шейдера превышает возможности GPU",
                    effect.CurrentTechnique.Name, effectFileName),
                    "Критическая ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Idle += new EventHandler(Application_Idle);
                return;
            }
        }
        
        void Application_Idle(object sender, EventArgs e)
        {
            this.Close();
        }
    
        private void xnaPanel1_Paint(object sender, PaintEventArgs e)
        {
            if (closing)
                return;
    
            try
            {
                if (device.GraphicsDeviceStatus == GraphicsDeviceStatus.Lost)
                    throw new DeviceLostException();
    
                if (device.GraphicsDeviceStatus == GraphicsDeviceStatus.NotReset)
                    device.Reset(presentParams);
    
                // Очищаем область экрана
                device.Clear(XNAGraphics.Color.CornflowerBlue);
    
                device.RenderState.CullMode = CullMode.None;
                device.VertexDeclaration = decl;
    
                // Задаем координаты вершин из установленного в группе панелей цвета
                vertices[0] = new VertexPositionColor(new Vector3(0.0f, 0.8f, 0.0f), new XNAGraphics.Color(
                    vertex1Color.BackColor.R, vertex1Color.BackColor.G, vertex1Color.BackColor.B));
                vertices[1] = new VertexPositionColor(new Vector3(0.8f, -0.8f, 0.0f), new XNAGraphics.Color(
                    vertex2Color.BackColor.R, vertex2Color.BackColor.G, vertex2Color.BackColor.B));
                vertices[2] = new VertexPositionColor(new Vector3(-0.8f, -0.8f, 0.0f), new XNAGraphics.Color(
                    vertex3Color.BackColor.R, vertex3Color.BackColor.G, vertex3Color.BackColor.B));
    
                effect.Begin();
                foreach(EffectPass pass in effect.CurrentTechnique.Passes)
                {
                    pass.Begin();
                    device.DrawUserPrimitives(PrimitiveType.TriangleList, vertices, 0, vertices.Length / 3);
                    pass.End();
                }
                effect.End();
    
                device.Present();
            }
            catch (DeviceNotResetException)
            {
                Invalidate();
            }
            catch (DeviceLostException)
            {
                closing = true;     // Больше не выполнять OnPaint()
                string title = "Сбой графического устройства";
                string message = "Работа программы будет завершена.\n"
                               + "Закройте все ненужные программы\n"
                               + "и повторите запуск этого приложения";
                MessageBox.Show(message, title, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                Application.Idle += new EventHandler(Application_Idle);
                return;
            }
            catch (Exception ext)           // Все другие исключения
            {
                closing = true;
                MessageBox.Show(ext.Message);
                Application.Idle += Application_Idle;
            }
        }
    
        private void xnaPanel1_Resize(object sender, EventArgs e)
        {
            // Устанавливаем размеры заднего буфера по клиентской области панели
            presentParams.BackBufferWidth = xnaPanel1.ClientSize.Width;
            presentParams.BackBufferHeight = xnaPanel1.ClientSize.Height;
            // Сбрасываем устройство и применяем к нему новые параметры  
            // Если окно не минимизировано
            if (this.WindowState != FormWindowState.Minimized)
                device.Reset(presentParams); 
        }
    
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            //  Удаляем (освобождаем) устройство
            device.Dispose();
            // На всякий случай присваиваем ссылке на устройство значение null 
            device = null;
        }
    
        private void vertexColor_Click(object sender, EventArgs e)
        {
            Button button = (Button)sender;
            colorDialog1.Color = button.BackColor;
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                button.BackColor = colorDialog1.Color;
                xnaPanel1.Invalidate();
            }
        }
    }
}
